@@ -14,7 +14,6 @@ val rpc
      | `Forbid_builds (** Promise not to build anything. For now, this isn't checked *)
      ]
 
-val signal_watcher : t -> [ `Yes | `No ]
 val watch_exclusions : t -> string list
 val stats : t -> Dune_stats.t option
 val print_metrics : t -> bool
@@ -24,7 +23,6 @@ val dump_memo_graph_with_timing : t -> bool
 val watch : t -> Dune_rpc_impl.Watch_mode_config.t
 val file_watcher : t -> Dune_engine.Scheduler.Run.file_watcher
 val prefix_target : t -> string -> string
-val insignificant_changes : t -> [ `React | `Ignore ]
 
 module Action_runner : sig
   type t =
@@ -49,8 +47,6 @@ module Builder : sig
   val term : t Cmdliner.Term.t
 end
 
-val build : Builder.t -> t
-
 (** [init] creates a [Common.t] by executing a sequence of side-effecting actions to
     initialize Dune's working environment based on the options determined in the\
     [Builder.t].
