@@ -406,7 +406,7 @@ let subst () =
   | Some kind -> Memo.run (subst { kind; root = Path.root })
 ;;
 
-(** A string that is "3.15.3" but not expanded by [dune subst] *)
+(** A string that is "3.16.0" but not expanded by [dune subst] *)
 let literal_version = "%%" ^ "VERSION%%"
 
 let doc = "Substitute watermarks in source files."
@@ -483,8 +483,7 @@ let term =
        config
        ~watch_exclusions:[]
        None
-       ~insignificant_changes:`React
-       ~signal_watcher:`No)
+       ~print_ctrl_c_warning:false)
     subst
 ;;
 
