@@ -6,9 +6,7 @@ end
 
 let report_errors_config = ref Report_errors_config.default
 let stop_on_first_error = ref false
-let debug_digests = ref false
 let debug_fs_cache = ref false
-let wait_for_filesystem_clock = ref false
 let capture_outputs = ref true
 
 let debug_backtraces b =
