@@ -159,12 +159,21 @@ let coq_env =
     |> Action_builder.of_memo_join
 ;;
 
-let coq_flags ~dir ~stanza_flags ~expander =
-  Expander.expand_and_eval_set
-    expander
-    stanza_flags
-    ~standard:
-      (Action_builder.map ~f:(fun { Coq_flags.coq_flags; _ } -> coq_flags) (coq_env ~dir))
+let coq_flags ~dir ~stanza_flags ~per_file_flags ~expander =
+  let standard, flags_to_expand =
+    match per_file_flags with
+    | None ->
+      ( Action_builder.map ~f:(fun { Coq_flags.coq_flags; _ } -> coq_flags) (coq_env ~dir)
+      , stanza_flags )
+    | Some per_file_flags ->
+      ( Action_builder.bind
+          ~f:(fun { Coq_flags.coq_flags; _ } ->
+            let standard = Action_builder.return coq_flags in
+            Expander.expand_and_eval_set expander stanza_flags ~standard)
+          (coq_env ~dir)
+      , per_file_flags )
+  in
+  Expander.expand_and_eval_set expander flags_to_expand ~standard
 ;;
 
 let coqdoc_flags ~dir ~stanza_coqdoc_flags ~expander =
@@ -200,15 +209,15 @@ module Bootstrap : sig
     | Stdlib of Coq_lib.t (** Regular case in >= 0.8 (or in < 0.8
                               (boot) was used *)
 
-  val empty : t Memo.t
+  val empty : t
 
   val make
-    :  dir:Path.Build.t
+    :  scope:Scope.t
     -> use_stdlib:bool
     -> wrapper_name:string
     -> coq_lang_version:Syntax.Version.t
     -> Coq_module.t
-    -> t Memo.t
+    -> t Resolve.Memo.t
 
   val flags : t -> 'a Command.Args.t
 end = struct
@@ -222,7 +231,7 @@ end = struct
 
   (* For empty set of modules, we return Prelude which is kinda
      conservative. *)
-  let empty = Memo.return No_stdlib
+  let empty = No_stdlib
 
   (* Hack to know if a module is a prelude module *)
   let check_init_module bootlib wrapper_name coq_module =
@@ -235,12 +244,12 @@ end = struct
 
   (* [Bootstrap.t] determines, for a concrete Coq module, how the Coq
      "standard library" is being handled. See the main modes above. *)
-  let make ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module =
-    let* scope = Scope.DB.find_by_dir dir in
-    let+ boot_lib =
+  let make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module =
+    let open Resolve.Memo.O in
+    let* boot_lib =
       Scope.coq_libs scope
+      |> Resolve.Memo.lift_memo
       >>= Coq_lib.DB.resolve_boot ~coq_lang_version
-      |> Resolve.Memo.read_memo
     in
     if use_stdlib
     then (
@@ -248,12 +257,16 @@ end = struct
       | None ->
         if coq_lang_version >= (0, 8)
         then
-          User_error.raise
-            [ Pp.text
-                "Couldn't find Coq standard library, and theory is not using (stdlib no)"
-            ]
-        else Implicit
+          Resolve.Memo.fail
+            (User_message.make
+               [ Pp.text
+                   "Couldn't find Coq standard library, and theory is not using (stdlib \
+                    no)"
+               ])
+        else Resolve.Memo.return Implicit
       | Some (_loc, boot_lib) ->
+        Resolve.Memo.return
+        @@
         (* TODO: replace with per_file flags *)
         let init = check_init_module boot_lib wrapper_name coq_module in
         if init
@@ -267,7 +280,7 @@ end = struct
           match boot_lib with
           | Legacy _ -> Implicit
           | Dune _ -> Stdlib boot_lib))
-    else No_stdlib
+    else Resolve.Memo.return No_stdlib
   ;;
 
   let flags t : _ Command.Args.t =
@@ -278,9 +291,7 @@ end = struct
   ;;
 end
 
-let coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
-  : _ Command.Args.t list
-  =
+let coqc_file_flags ~dir ~theories_deps ~wrapper_name ~ml_flags : _ Command.Args.t list =
   let file_flags : _ Command.Args.t list =
     [ Dyn (Resolve.Memo.read ml_flags)
     ; theories_flags ~theories_deps
@@ -289,7 +300,7 @@ let coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
     ; A wrapper_name
     ]
   in
-  [ Bootstrap.flags boot_type; S file_flags ]
+  [ S file_flags ]
 ;;
 
 let native_includes ~dir =
@@ -459,30 +470,21 @@ let setup_coqdep_for_theory_rule
   ~loc
   ~theories_deps
   ~wrapper_name
-  ~use_stdlib
   ~source_rule
   ~ml_flags
   ~mlpack_rule
-  ~coq_lang_version
+  ~boot_flags
   coq_modules
   =
-  let* boot_type =
-    (* If coq_modules are empty it doesn't really matter, so we take
-       the more conservative path and pass -boot, we don't care here
-       about -noinit as coqdep ignores it *)
-    match coq_modules with
-    | [] -> Bootstrap.empty
-    | m :: _ -> Bootstrap.make ~dir ~use_stdlib ~wrapper_name ~coq_lang_version m
-  in
   (* coqdep needs the full source + plugin's mlpack to be present :( *)
   let sources = List.rev_map ~f:Coq_module.source coq_modules in
   let file_flags =
-    [ Command.Args.S
-        (coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags)
+    [ Command.Args.S (coqc_file_flags ~dir ~theories_deps ~wrapper_name ~ml_flags)
     ; As [ "-dyndep"; "opt"; "-vos" ]
     ; Deps sources
     ]
   in
+  let coqdep_flags = Command.Args.Dyn boot_flags :: file_flags in
   let stdout_to = dep_theory_file ~dir ~wrapper_name in
   let* coqdep =
     Super_context.resolve_program_memo
@@ -501,7 +503,7 @@ let setup_coqdep_for_theory_rule
     (let open Action_builder.With_targets.O in
      Action_builder.with_no_targets mlpack_rule
      >>> Action_builder.(with_no_targets (goal source_rule))
-     >>> Command.run ~dir:(Path.build dir) ~stdout_to coqdep file_flags)
+     >>> Command.run ~dir:(Path.build dir) ~stdout_to coqdep coqdep_flags)
 ;;
 
 module Dep_map = Stdune.Map.Make (Path)
@@ -519,7 +521,8 @@ let coqdep_invalid phase line =
    Note that coqdep escapes a few more things, including spaces, $, #,
    [], ?, %, homedir... How to handle that seems tricky.
 *)
-let unescape_coqdep string = Re.replace_string (Re.compile (Re.str "\\:")) ~by:":" string
+let escaped_colon = Re.compile (Re.str "\\:")
+let unescape_coqdep string = Re.replace_string escaped_colon ~by:":" string
 
 let parse_line ~dir line =
   match String.lsplit2 line ~on:':' with
@@ -563,6 +566,26 @@ let get_dep_map ~dir ~wrapper_name : Path.t list Dep_map.t Action_builder.t =
       ]
 ;;
 
+(* EJGA: Would be nice to have a functor in [Stdune.Tuple] to do this *)
+module DWInput : Memo.Input with type t = Path.Build.t * string = struct
+  type t = Path.Build.t * string
+
+  let hash = Tuple.T2.hash Path.Build.hash String.hash
+  let equal = Tuple.T2.equal Path.Build.equal String.equal
+  let to_dyn = Tuple.T2.to_dyn Path.Build.to_dyn String.to_dyn
+end
+
+(* EJGA: with this code, we have a combined memo node that will be
+   re-executed iff [dir], [wrapper_name], or the deps of the original
+   action builder (the [.v.d] file from [dep_theory_file ~dir
+   ~wrapper_name]) change *)
+let memo_get_dep_map =
+  Action_builder.create_memo
+    "coq_dep_map"
+    ~input:(module DWInput)
+    (fun (dir, wrapper_name) -> get_dep_map ~dir ~wrapper_name)
+;;
+
 let deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module =
   let open Action_builder.O in
   let vo_target =
@@ -573,7 +596,8 @@ let deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module =
     in
     Path.set_extension ~ext (Coq_module.source coq_module)
   in
-  let* dep_map = get_dep_map ~dir ~wrapper_name in
+  let* dep_map = Action_builder.exec_memo memo_get_dep_map (dir, wrapper_name) in
+  let* boot_type = Resolve.Memo.read boot_type in
   match Dep_map.find dep_map vo_target with
   | None ->
     Code_error.raise
@@ -596,7 +620,8 @@ let generic_coq_args
   ~sctx
   ~dir
   ~wrapper_name
-  ~boot_type
+  ~boot_flags
+  ~per_file_flags
   ~mode
   ~coq_prog
   ~stanza_flags
@@ -608,7 +633,7 @@ let generic_coq_args
   let+ coq_stanza_flags =
     let+ expander = Super_context.expander sctx ~dir in
     let coq_flags =
-      let coq_flags = coq_flags ~expander ~dir ~stanza_flags in
+      let coq_flags = coq_flags ~expander ~dir ~stanza_flags ~per_file_flags in
       (* By default we have the -q flag. We don't want to pass this to coqtop to
          allow users to load their .coqrc files for interactive development.
          Therefore we manually scrub the -q setting when passing arguments to
@@ -635,26 +660,36 @@ let generic_coq_args
     in
     coqc_native_flags ~sctx ~dir ~theories_deps ~theory_dirs ~mode
   in
-  let file_flags =
-    coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
-  in
+  let file_flags = coqc_file_flags ~dir ~theories_deps ~wrapper_name ~ml_flags in
   match coq_prog with
   | `Coqc ->
     [ coq_stanza_flags
     ; coq_native_flags
+    ; Dyn boot_flags
     ; S file_flags
     ; Dep (Coq_module.source coq_module)
     ]
-  | `Coqtop -> [ coq_stanza_flags; coq_native_flags; S file_flags ]
+  | `Coqtop -> [ coq_stanza_flags; coq_native_flags; Dyn boot_flags; S file_flags ]
 ;;
 
+module Per_file = struct
+  let match_ map coq_module =
+    let open Option.O in
+    let* map = map in
+    let coq_lib_name = Coq_module.name coq_module in
+    Coq_module.Name.Map.find map coq_lib_name
+  ;;
+end
+
 let setup_coqc_rule
+  ~scope
   ~loc
   ~dir
   ~sctx
   ~coqc_dir
   ~file_targets
   ~stanza_flags
+  ~modules_flags
   ~theories_deps
   ~mode
   ~wrapper_name
@@ -665,9 +700,12 @@ let setup_coqc_rule
   coq_module
   =
   (* Process coqdep and generate rules *)
-  let* boot_type =
-    Bootstrap.make ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
+  let boot_type =
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  (* TODO: merge with boot_type *)
+  let per_file_flags = Per_file.match_ modules_flags coq_module in
   let* coqc = coqc ~loc ~dir ~sctx in
   let obj_files =
     Coq_module.obj_files
@@ -684,7 +722,8 @@ let setup_coqc_rule
       ~sctx
       ~dir
       ~wrapper_name
-      ~boot_type
+      ~boot_flags
+      ~per_file_flags
       ~stanza_flags
       ~ml_flags
       ~theories_deps
@@ -890,6 +929,7 @@ let setup_theory_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) =
   let name = snd s.name in
   let loc = s.buildable.loc in
   let stanza_flags = s.buildable.flags in
+  let modules_flags = Option.map s.modules_flags ~f:Coq_module.Name.Map.of_list_exn in
   let* coq_dir_contents = Dir_contents.coq dir_contents in
   let theory_dirs =
     Coq_sources.directories coq_dir_contents ~name |> Path.Build.Set.of_list
@@ -909,27 +949,37 @@ let setup_theory_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) =
   let coqc_dir = Super_context.context sctx |> Context.build_dir in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   (* First we setup the rule calling coqdep *)
+  let boot_type =
+    (* If coq_modules are empty it doesn't really matter, so we take
+       the more conservative path and pass -boot, we don't care here
+       about -noinit as coqdep ignores it *)
+    match coq_modules with
+    | [] -> Resolve.Memo.return Bootstrap.empty
+    | m :: _ -> Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version m
+  in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
   setup_coqdep_for_theory_rule
     ~sctx
     ~dir
     ~loc
     ~theories_deps
     ~wrapper_name
-    ~use_stdlib
     ~source_rule
     ~ml_flags
     ~mlpack_rule
-    ~coq_lang_version
+    ~boot_flags
     coq_modules
   >>> Memo.parallel_iter
         coq_modules
         ~f:
           (setup_coqc_rule
+             ~scope
              ~loc
              ~dir
              ~sctx
              ~file_targets:[]
              ~stanza_flags
+             ~modules_flags
              ~coqc_dir
              ~theories_deps
              ~mode
@@ -954,18 +1004,21 @@ let coqtop_args_theory ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) coq_mo
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   let name = snd s.name in
   let use_stdlib = s.buildable.use_stdlib in
-  let* boot_type =
-    Bootstrap.make ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
+  let boot_type =
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
   let* coq_dir_contents = Dir_contents.coq dir_contents in
   let theory_dirs =
     Coq_sources.directories coq_dir_contents ~name |> Path.Build.Set.of_list
   in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  let per_file_flags = None in
   generic_coq_args
     ~sctx
     ~dir
     ~wrapper_name
-    ~boot_type
+    ~boot_flags
+    ~per_file_flags
     ~mode
     ~coq_prog:`Coqtop
     ~stanza_flags:s.buildable.flags
@@ -1116,21 +1169,32 @@ let setup_extraction_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Extraction.t
     source_rule ~sctx theories_deps >>> Action_builder.path (Coq_module.source coq_module)
   in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
+  let boot_type =
+    (* If coq_modules are empty it doesn't really matter, so we take
+       the more conservative path and pass -boot, we don't care here
+       about -noinit as coqdep ignores it *)
+    match [ coq_module ] with
+    | [] -> Resolve.Memo.return Bootstrap.empty
+    | m :: _ -> Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version m
+  in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  let modules_flags = None in
   setup_coqdep_for_theory_rule
     ~sctx
     ~dir
     ~loc
     ~theories_deps
     ~wrapper_name
-    ~use_stdlib
     ~source_rule
     ~ml_flags
     ~mlpack_rule
-    ~coq_lang_version
+    ~boot_flags
     [ coq_module ]
   >>> setup_coqc_rule
+        ~scope
         ~file_targets
         ~stanza_flags:s.buildable.flags
+        ~modules_flags
         ~sctx
         ~loc
         ~coqc_dir:dir
@@ -1154,15 +1218,18 @@ let coqtop_args_extraction ~sctx ~dir (s : Coq_stanza.Extraction.t) coq_module =
     extraction_context ~context ~scope ~coq_lang_version s.buildable
   in
   let wrapper_name = "DuneExtraction" in
-  let* boot_type =
-    Bootstrap.make ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
+  let boot_type =
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  let per_file_flags = None in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   generic_coq_args
     ~sctx
     ~dir
     ~wrapper_name
-    ~boot_type
+    ~boot_flags
+    ~per_file_flags
     ~mode
     ~coq_prog:`Coqtop
     ~stanza_flags:s.buildable.flags
@@ -1174,10 +1241,10 @@ let coqtop_args_extraction ~sctx ~dir (s : Coq_stanza.Extraction.t) coq_module =
 
 (* Version for export *)
 let deps_of ~dir ~use_stdlib ~wrapper_name ~mode ~coq_lang_version coq_module =
-  let open Action_builder.O in
-  let* boot_type =
-    Action_builder.of_memo
-      (Bootstrap.make ~dir ~use_stdlib ~wrapper_name ~coq_lang_version coq_module)
+  let boot_type =
+    let open Memo.O in
+    let* scope = Scope.DB.find_by_dir dir in
+    Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
   deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module
 ;;
