@@ -413,7 +413,9 @@ let setup_library_odoc_rules cctx (local_lib : Lib.Local.t) =
     in
     Dep.deps ctx package requires, odoc_include_flags
   in
-  Modules.fold_no_vlib modules ~init:[] ~f:(fun m acc ->
+  modules
+  |> Modules.With_vlib.drop_vlib
+  |> Modules.fold ~init:[] ~f:(fun m acc ->
     let compiled =
       let modes = Lib_info.modes info in
       let mode = Lib_mode.Map.Set.for_merlin modes in
@@ -590,7 +592,9 @@ let libs_of_pkg ctx ~pkg =
 ;;
 
 let entry_modules_by_lib sctx lib =
-  Dir_contents.modules_of_local_lib sctx lib >>| Modules.entry_modules
+  Dir_contents.modules_of_local_lib sctx lib
+  >>| Modules.With_vlib.modules
+  >>| Modules.With_vlib.entry_modules
 ;;
 
 let entry_modules sctx ~pkg =
@@ -939,7 +943,8 @@ let setup_package_odoc_rules sctx ~pkg =
 
 let gen_project_rules sctx project =
   Dune_project.packages project
-  |> Package_map_traversals.parallel_iter ~f:(fun _ (pkg : Package.t) ->
+  |> Dune_lang.Package_name.Map.to_seq
+  |> Memo.parallel_iter_seq ~f:(fun (_, (pkg : Package.t)) ->
     (* setup @doc to build the correct html for the package *)
     setup_package_aliases sctx pkg)
 ;;
@@ -950,7 +955,10 @@ let setup_private_library_doc_alias sctx ~scope ~dir (l : Library.t) =
   | Private _ ->
     let ctx = Super_context.context sctx in
     let* lib =
-      Lib.DB.find_even_when_hidden (Scope.libs scope) (Library.best_name l)
+      let src_dir = Path.drop_optional_build_context_src_exn (Path.build dir) in
+      Lib.DB.find_lib_id_even_when_hidden
+        (Scope.libs scope)
+        (Local (Library.to_lib_id ~src_dir l))
       >>| Option.value_exn
     in
     let lib = Lib (Lib.Local.of_lib_exn lib) in
