@@ -10,7 +10,7 @@ module Lock_dir : sig
     ; unset_solver_vars : Dune_lang.Package_variable_name.Set.t option
     ; repositories : (Loc.t * Dune_pkg.Pkg_workspace.Repository.Name.t) list
     ; constraints : Dune_lang.Package_dependency.t list
-    ; sources : string list
+    ; pins : (Loc.t * string) list
     }
 
   val equal : t -> t -> bool
@@ -38,6 +38,16 @@ module Context : sig
     val equal : t -> t -> bool
   end
 
+  module Merlin : sig
+    type t =
+      | Selected
+      | Rules_only
+      | Not_selected
+
+    val equal : t -> t -> bool
+    val to_dyn : t -> Dyn.t
+  end
+
   module Common : sig
     type t =
       { loc : Loc.t
@@ -57,7 +67,7 @@ module Context : sig
           the runtime system. *)
       ; dynamically_linked_foreign_archives : bool
       ; instrument_with : Lib_name.t list
-      ; merlin : bool
+      ; merlin : Merlin.t
       }
   end
 
@@ -107,7 +117,7 @@ type t = private
   ; repos : Dune_pkg.Pkg_workspace.Repository.t list
   ; lock_dirs : Lock_dir.t list
   ; dir : Path.Source.t
-  ; sources : Dune_pkg.Pin_stanza.DB.t
+  ; sources : Dune_pkg.Pin_stanza.DB.Workspace.t
   }
 
 val equal : t -> t -> bool
